# -*- coding: utf-8 -*-

from __future__ import division

__copyright__ = """
Copyright (c) 2015-2016 Dong Zhuang (dzhuang.scut@gmail.com)
"""

__license__ = """
The MIT License (MIT)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the 
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so, subject to the 
following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from django.conf import settings

from email.header import Header
from email.utils import parseaddr

from django.utils.encoding import force_text

def split_addr_name(addr, encoding):
    if not isinstance(addr, tuple):
        addr = parseaddr(force_text(addr))
    nm, addr = addr
    nm = Header(nm, encoding).encode()
    try:
        addr.encode('ascii')
    except UnicodeEncodeError:  # IDN
        if '@' in addr:
            localpart, domain = addr.split('@', 1)
            localpart = str(Header(localpart, encoding))
            domain = domain.encode('idna').decode('ascii')
            addr = '@'.join([localpart, domain])
        else:
            addr = Header(addr, encoding).encode()
    return addr, nm

def make_config(email):
    """
    Make config from headers or settings.py
    """
    appid = None
    appkey = None
    if email.extra_headers:
        if "SUBMAIL_APP_KEY" in email.extra_headers:
            appkey = email.extra_headers["SUBMAIL_APP_KEY"]
        if "SUBMAIL_APP_ID" in email.extra_headers:
            appid = email.extra_headers["SUBMAIL_APP_ID"]
    if appkey is None:
        appkey = getattr(settings, "SUBMAIL_APP_KEY", None)
    if appid is None:
        appid = getattr(settings, "SUBMAIL_APP_ID", None)
    sign_type = getattr(settings, "SUBMAIL_SIGN_TYPE", "normal")

    config = {}
    if appid and appkey:
        config['appid'] = appid
        config['appkey'] = appkey
        config['sign_type'] = sign_type
    else:
        from django.core.exceptions import ImproperlyConfigured
        raise ImproperlyConfigured(
            "Both SUBMAIL_APP_ID and SUBMAIL_APP_KEY must be "
            "declared in settings.py or in your EmailMessage headers.")
    return config

def make_django_email_subject(s):
    if len(s) <= 100:
        return s
    return s[:97] + "..."




import hashlib
from six.moves.urllib.request import urlopen  # noqa
from six.moves.urllib.parse import urlencode  # noqa


class Message:
    '''
    @init
    '''
    def __init__(self, message_configs):
        self.sign_type = 'normal'
        self.message_configs = message_configs
        self.signature = ''
    
    '''
    @createSignature
    '''
    def __create_signature(self, request):
        if self.sign_type == 'normal':
            self.signature = self.message_configs['appkey']
        else:
            self.__build_signature(request)

    '''
    @buildSignature
    '''
    def __build_signature(self, request):
        appid = self.message_configs['appid']
        appkey = self.message_configs['appkey']
        para_keys = request.keys();
        para_keys.sort()
        sign_str = ''
        for key in para_keys:
            sign_str += "%s=%s&"%(key,request[key])
        sign_str = appid+appkey+sign_str[:-1]+appid+appkey
        if self.sign_type == 'md5':
            hash=hashlib.md5()
            hash.update(sign_str)
            self.signature = hash.hexdigest()
        elif self.sign_type == 'sha1':
            hash=hashlib.sha1() 
            hash.update(sign_str)
            self.signature = hash.hexdigest()

    def __http_get(self,url):
        return eval(urlopen(url = url).read())

    def __http_post(self, url, para):
        data = urlencode(para)
        resp = urlopen(url=url,data = data.encode()).read()
        return eval(resp)
         
    '''
    @getTimestamp
    '''
    def get_timestamp(self):
        api = 'https://api.submail.cn/service/timestamp.json'
        resp = self.__http_get(api)
        return resp['timestamp']

    '''
    @Send
    '''
    def send(self,request):
        '''
        @setup API httpRequest URI
        '''
        api = 'https://api.submail.cn/message/send.json'

        '''
        create final API post query Start
        '''
        request['appid'] = self.message_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()

        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if 'sign_type' not in self.message_configs:
            self.sign_type = 'normal'
        elif self.message_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.mail_config['sign_type'] 
            request['sign_type'] = self.mail_config['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature
        
        '''
        create final API post query End
        '''

        '''
        @send request
        '''
        return self.__http_post(api, request)
        
    '''
    @xsend
    '''
    def xsend(self, request):
        '''
        @setup API httpRequest URI
        '''
        api = 'https://api.submail.cn/message/xsend.json'

        '''
        create final API post query Start
        '''
        request['appid'] = self.message_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()

        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if 'sign_type' not in self.message_configs:
            self.sign_type = 'normal'
        elif self.message_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.message_configs['sign_type'] 
            request['sign_type'] = self.message_configs['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature
        '''
        create final API post query End
        '''

        '''
        @send request
        '''
        return self.__http_post(api, request)

    '''
    @addressbook/message/subscribe
    '''
    def subscribe(self,request):
        '''
        @setup API httpRequest URI
        '''
        api='https://api.submail.cn/addressbook/message/subscribe.json'

        '''
        create final API post query Start
        '''
        request['appid'] = self.message_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()
        
        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if 'sign_type' not in self.message_configs:
            self.sign_type = 'normal'
        elif self.message_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.message_configs['sign_type'] 
            request['sign_type'] = self.message_configs['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature
        
        '''
        create final API post query End
        '''

        '''
        @subscribe request
        '''
        return self.__http_post(api, request)

    '''
    @addressbook/message/unsubscribe
    '''
    def unsubscribe(self,request):
        '''
        @setup API httpRequest URI
        '''
        api='https://api.submail.cn/addressbook/message/unsubscribe.json'

        '''
        create final API post query Start
        '''
        request['appid'] = self.message_configs['appid']

        '''
        @get timestamp from server
        '''
        request['timestamp'] = self.get_timestamp()
        
        '''
        @setup sign_type
        '''
        sign_type_state = ['normal','md5','sha1']
        if 'sign_type' not in self.message_configs:
            self.sign_type = 'normal'
        elif self.message_configs['sign_type'] not in sign_type_state:
            self.sign_type = 'normal'
        else:
            self.sign_type = self.message_configs['sign_type'] 
            request['sign_type'] = self.message_configs['sign_type']

        '''
        @create signature
        '''
        self.__create_signature(request)
        request['signature'] = self.signature

        '''
        create final API post query End
        '''

        '''
        @unsubscribe request
        '''
        return self.__http_post(api, request)




from .mail import Mail
import json

class MAILXsend:
    '''
    @set vars start
    '''
    def __init__(self, configs):
        '''
        Submail mail appid
        @type string
        '''
        self.appid = configs['appid']

        '''
        Submail mail appkey
        @type string
        '''
        self.appkey = configs['appkey']

        '''
        sign type (Optional)
        @options: normal or md5 or sha1
        '''
        self.sign_type = ''
        if 'sign_type' in configs:
            self.sign_type = configs['sign_type']

        '''
        to: email recipient
        @array to rfc 822
        '''
        self.to = []

        '''
        add email recipient from addressbook
        @array to string
        '''
        self.address_book = []

        '''
        Sender email
        @type email
        '''
        self.from_ = ''

        '''
        Sender display name
        @type string
        '''
        self.from_name = ''

        '''
        Sender reply email
        @type email
        '''
        self.reply = ''

        '''
        cc recipient email
        @array to string
        '''
        self.cc = []
        
        '''
        bcc recipient email
        @array to string
        '''
        self.bcc = []

        '''
        the email subject
        @string
        '''
        self.subject = ''

        '''
        email project sign
        '''
        self.project = ''

        '''
        vars: the submail email text content filter
        @type array to json string
        '''
        self.vars = {}

        '''
        links: the submail email links content filter
        @type array to json string
        '''
        self.links = {}

        '''
        email headers
        @type  array to json string
        '''
        self.headers = {}
    
    '''
    addTo function
    set email and name to array
    '''
    def add_to(self, address, name=''):
        self.to.append({'address':address, 'name':name})

    '''
    AddAddressbook function
    set addressbook sign to array
    '''
    def add_address_book(self,address_book):
        self.address_book.append(address_book)

    '''
    setSender function
    set From and From_name
    '''
    def set_sender(self,sender,name=''):
        self.from_ = sender
        self.from_name = name

    '''
    SetReply function
    set Reply address
    '''
    def set_reply(self, reply):
        self.reply = reply

    '''
    AddCc function
    set cc recipient to array
    '''
    def add_cc(self, address, name=''):
        self.cc.append({'address':address, 'name':name})

    '''
    AddBcc function
    set bcc recipient to array
    '''
    def add_bcc(self, address, name=''):
        self.bcc.append({'address':address, 'name':name})
    
    '''
    Set email subject
    '''
    def set_subject(self, subject):
        self.subject = subject

    '''
    Set email project
    '''
    def set_project(self, project):
        self.project = project

    '''
    AddVar function
    set var to array
    '''
    def add_var(self, key, val):
        self.vars[key] = val

    '''
    AddLink function
    set link var to array
    '''
    def add_link(self, key, val):
        self.links[key] = val
    
    '''
    AddHeaders function
    set headers to array
    '''
    def add_header(self, key, val):
        self.headers[key] = val

    '''
    build request array
    '''
    def build_request(self):
        request = {}
        '''
        convert To array to rfc 822 format
        '''
        if len(self.to) != 0:
            request['to'] = ''
            for to in self.to:
                request['to'] += to['name']+'<'+to['address']+'>,'
            request['to'] = request['to'][:-1]
        
        '''
        convert Addressbook array to string
        '''
        if len(self.address_book) != 0:
            request['addressbook'] = 0
            for address_book in self.address_book:
                request['addressbook'] += address_book+','
            request['addressbook'] = request['addressbook'][:-1]

        '''
        set sender email
        '''
        if self.from_ != '':
            request['from'] = self.from_

        '''
        set sender display name, if is not empty
        '''
        if self.from_name != '':
            request['from_name'] = self.from_name

        '''
        set sender reply email, if is not empty
        '''
        if self.reply != '':
            request['reply'] = self.reply

        '''
        convert Cc array to rfc 822 format, if is not empty
        '''
        if len(self.cc) != 0:
            request['cc'] = ''
            for cc in self.cc:
                request['cc'] += cc['name']+'<'+cc['address']+'>,'
            request['cc'] = request['cc'][:-1]

        '''
        convert Bcc array to rfc 822 format, if is not empty
        '''
        if len(self.bcc) != 0:
            request['bcc'] = ''
            for bcc in self.bcc:
                request['bcc'] += bcc['name']+'<'+bcc['address']+'>,'
            request['bcc'] = request['bcc'][:-1]

        '''
        set email subject
        '''
        if self.subject != '':
            request['subject'] = self.subject

        '''
        set project sign
        '''
        request['project'] = self.project

        '''
        convert vars array to json string, if is not empty
        '''
        if len(self.vars) != 0:
            request['vars'] = json.dumps(self.vars)

        '''
        convert links array to json string, if is not empty
        '''
        if len(self.links) != 0:
            request['links'] = json.dumps(self.links)

        '''
        convert Headers array to json string, if is not empty
        '''
        if len(self.headers) != 0:
            request['headers'] = json.dumps(self.headers)
        return request

    '''
    xsend email
    '''
    def xsend(self):
        mail_configs = {}
        '''
        set appid and appkey
        '''
        mail_configs['appid'] = self.appid
        mail_configs['appkey'] = self.appkey

        '''
        set sign_type,if is set
        '''
        if self.sign_type != '':
            mail_configs['sign_type'] = self.sign_type

        '''
        init mail class
        '''
        mail = Mail(mail_configs)

        '''
        build request and send email and return the result
        '''
        return mail.xsend(self.build_request())



# -*- coding: utf-8 -*-

from __future__ import division

__copyright__ = """
Copyright (c) 2015-2016 Dong Zhuang (dzhuang.scut@gmail.com)
"""

__license__ = """
The MIT License (MIT)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the 
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to permit
persons to whom the Software is furnished to do so, subject to the 
following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail import EmailMultiAlternatives
from email.utils import parseaddr

from .submail.mail_send import MAILSend

from .utils import split_addr_name, make_config, make_django_email_subject


class SubmailBackend(BaseEmailBackend):
    '''
    Django submail mail/send Backend
    '''

    def open(self):
        pass

    def close(self):
        pass

    def send_messages(self, emails):
        if not emails:
            return

        result = 0
        for email in emails:
            mail = build_sm_mail(email)
            response = mail.send()
            if response["status"] == "success":
                result += 1
        if result:
            return result
        else:
            return None


def build_sm_mail(email):
    """
    Convert django email class to submail.MAILSend class
    """
    mail = MAILSend(make_config(email))

    from_addr, from_name = split_addr_name(
        email.from_email, email.encoding)
    mail.set_sender(from_addr, from_name)

    for send_to in email.to:
        to_addr, to_name = split_addr_name(send_to, email.encoding)
        mail.add_to(to_addr, to_name)

    for send_cc in email.cc:
        cc_addr, cc_name = split_addr_name(send_cc, email.encoding)
        mail.add_cc(cc_addr, cc_name)

    for bcc in email.bcc:
        bcc_addr, bcc_name = split_addr_name(bcc, email.encoding)
        mail.add_bcc(bcc_addr, bcc_name)

    if email.reply_to:
        mail.set_reply(parseaddr(email.reply_to)[1])
    elif email.extra_headers:
        if "Reply-To" in email.extra_headers:
            mail.set_reply(parseaddr(email.extra_headers["Reply-To"])[1])

    # if the email is have "reply-to" and the recepient is a qq.com email
    # add notification not to reply at mobile clients.
    has_qqmail_recepient = False
    for e in email.to:
        if "@qq.com" in e:
            has_qqmail_recepient = True
            break

    qqmail_reply_to_html_alert = (
        u'<p class="netdisk_hide"><strong>'
        u'注意：QQ邮箱用户请勿使用手机客户端'
        u'或微信客户端回复本邮件，请使用pc客户端回复.'
        '</strong></p>')

    if len(email.reply_to) > 0 and has_qqmail_recepient:
        if email.body:
            body = email.body
            lines = body.splitlines()
            html_body = "".join("<p>%s</p>" % line for line in lines)
            html_body += qqmail_reply_to_html_alert
            mail.set_html(html_body)
    else:
        mail.set_text(email.body)

    subject = make_django_email_subject(email.subject)
    mail.set_subject(subject)

    if isinstance(email, EmailMultiAlternatives):
        for alt in email.alternatives:
            if alt[1] == "text/html":
                if email.reply_to is not None and has_qqmail_recepient:
                    mail.set_html(alt[0] + qqmail_reply_to_html_alert)
                else:
                    mail.set_html(alt[0])

    for attachment in email.attachments:
        mail.add_attachment(attachment)

    return mail



from .message import Message
import json

class MESSAGEXsend:
    '''
    @set vars start
    '''
    def __init__(self, configs):
        '''
        Submail mail appid
        @type string
        '''
        self.appid = configs['appid']

        '''
        Submail mail appkey
        @type string
        '''
        self.appkey = configs['appkey']

        '''
        sign type (Optional)
        @options: normal or md5 or sha1
        '''
        self.sign_type = ''
        if 'sign_type' in configs:
            self.sign_type = configs['sign_type']

        '''
        to: email recipient
        @array to rfc 822
        '''
        self.to = []

        '''
        add email recipient from addressbook
        @array to string
        '''
        self.address_book = []

        '''
        email project sign
        '''
        self.project = ''

        '''
        vars: the submail email text content filter
        @type array to json string
        '''
        self.vars = {}

    '''
    addTo function
    add message cellphone number
    '''
    def add_to(self, address):
        self.to.append(address)

    '''
    AddAddressbook function
    set addressbook sign to array
    '''
    def add_address_book(self,address_book):
        self.address_book.append(address_book)

    '''
    Set email project
    '''
    def set_project(self, project):
        self.project = project

    '''
    AddVar function
    set var to array
    '''
    def add_var(self, key, val):
        self.vars[key] = val

    '''
    build request array
    '''
    def build_request(self):
        request = {}
        '''
        convert To array to rfc 822 format
        '''
        if len(self.to) != 0: 
            request['to'] = ''
            for to in self.to:
                request['to'] += to + ','
            request['to'] = request['to'][:-1]
        
        '''
        convert Addressbook array to string
        '''
        if len(self.address_book) != 0:
            request['addressbook'] = '' 
            for address_book in self.address_book:
                request['addressbook'] += address_book+','
            request['addressbook'] = request['addressbook'][:-1]
      
        '''
        set project sign
        '''
        request['project'] = self.project

        '''
        convert vars array to json string, if is not empty
        '''
        if len(self.vars) != 0:
            request['vars'] = json.dumps(self.vars)

        return request
     
    '''
    xsend email
    '''
    def xsend(self):
        '''
        set appid and appkey
        '''
        message_configs = {}
        message_configs['appid'] = self.appid
        message_configs['appkey'] = self.appkey

        '''
        set sign_type,if is set
        '''
        if self.sign_type != '':
            message_configs['sign_type'] = self.sign_type

        '''
        init mail class
        '''
        message = Message(message_configs)

        '''
        build request and send email and return the result
        '''
        return message.xsend(self.build_request())



'''
Submail addressbook/message/unsubscribe API demo
SUBMAIL SDK Version 1.0.1 --python
copyright 2011 - 2014 SUBMAIL
'''
from .app_configs import MESSAGE_CONFIGS
from .address_book_message import ADDRESSBOOKMessage
'''
init MESSAGEXsend class
'''
addressbook = ADDRESSBOOKMessage(MESSAGE_CONFIGS)

'''
Required para
The First para: recipient email address
The second para: recipient name(optional)
'''
addressbook.set_address('18616761889')

'''
Optional para
set target addressbook sign : Optional
default value: subscribe
'''
#addressbook.set_address_book('subscribe')
print(addressbook.subscribe())



from setuptools import setup, find_packages
import smbackend

setup(
    name='django-submail',
    version=smbackend.__version__,
    author=smbackend.__author__,
    author_email='dzhuang.scut@gmail.com',
    keywords="django, submail, backend",
    url='https://github.com/dzhuang/django-submail',
    packages=find_packages(exclude=["demo"]),
    include_package_data=True,
    license=smbackend.__licence__,
    description=smbackend.__desc__,
    long_description=open('./README.rst').read(),
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        "License :: OSI Approved :: %s" % smbackend.__licence__,
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    )
